// SPDX-FileCopyrightText: © 2024 Allegro DVT <github-ip@allegrodvt.com>
// SPDX-License-Identifier: MIT

#include "convert_module_soft_hevc.h"

HEVCProfileType ConvertSoftToModuleHEVCHighTierProfile(AL_EProfile const& profile)
{
  if(!AL_IS_HEVC(profile))
    return HEVCProfileType::HEVC_PROFILE_MAX_ENUM;
  switch(profile)
  {
  case AL_PROFILE_HEVC_MAIN: return HEVCProfileType::HEVC_PROFILE_MAIN_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN10: return HEVCProfileType::HEVC_PROFILE_MAIN_10_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_STILL: return HEVCProfileType::HEVC_PROFILE_MAIN_STILL_HIGH_TIER;
  case AL_PROFILE_HEVC_MONO: return HEVCProfileType::HEVC_PROFILE_MONOCHROME_HIGH_TIER;
  case AL_PROFILE_HEVC_MONO10: return HEVCProfileType::HEVC_PROFILE_MONOCHROME_10_HIGH_TIER;
  case AL_PROFILE_HEVC_MONO12: return HEVCProfileType::HEVC_PROFILE_MONOCHROME_12_HIGH_TIER;
  case AL_PROFILE_HEVC_MONO16: return HEVCProfileType::HEVC_PROFILE_MONOCHROME_16_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN12: return HEVCProfileType::HEVC_PROFILE_MAIN_12_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_422: return HEVCProfileType::HEVC_PROFILE_MAIN_422_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_422_10: return HEVCProfileType::HEVC_PROFILE_MAIN_422_10_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_422_12: return HEVCProfileType::HEVC_PROFILE_MAIN_422_12_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_444: return HEVCProfileType::HEVC_PROFILE_MAIN_444_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_444_10: return HEVCProfileType::HEVC_PROFILE_MAIN_444_10_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_444_12: return HEVCProfileType::HEVC_PROFILE_MAIN_444_12_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_INTRA_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN10_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_10_INTRA_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN12_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_12_INTRA_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_422_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_422_INTRA_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_422_10_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_422_10_INTRA_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_422_12_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_422_12_INTRA_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_444_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_444_INTRA_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_444_10_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_444_10_INTRA_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_444_12_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_444_12_INTRA_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_444_16_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_444_16_INTRA_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_444_STILL: return HEVCProfileType::HEVC_PROFILE_MAIN_444_STILL_HIGH_TIER;
  case AL_PROFILE_HEVC_MAIN_444_16_STILL: return HEVCProfileType::HEVC_PROFILE_MAIN_444_16_STILL_HIGH_TIER;
  default: return HEVCProfileType::HEVC_PROFILE_MAX_ENUM;
  }

  return HEVCProfileType::HEVC_PROFILE_MAX_ENUM;
}

HEVCProfileType ConvertSoftToModuleHEVCMainTierProfile(AL_EProfile const& profile)
{
  if(!AL_IS_HEVC(profile))
    return HEVCProfileType::HEVC_PROFILE_MAX_ENUM;
  switch(profile)
  {
  case AL_PROFILE_HEVC_MAIN: return HEVCProfileType::HEVC_PROFILE_MAIN;
  case AL_PROFILE_HEVC_MAIN10: return HEVCProfileType::HEVC_PROFILE_MAIN_10;
  case AL_PROFILE_HEVC_MAIN_STILL: return HEVCProfileType::HEVC_PROFILE_MAIN_STILL;
  case AL_PROFILE_HEVC_MONO: return HEVCProfileType::HEVC_PROFILE_MONOCHROME;
  case AL_PROFILE_HEVC_MONO10: return HEVCProfileType::HEVC_PROFILE_MONOCHROME_10;
  case AL_PROFILE_HEVC_MONO12: return HEVCProfileType::HEVC_PROFILE_MONOCHROME_12;
  case AL_PROFILE_HEVC_MONO16: return HEVCProfileType::HEVC_PROFILE_MONOCHROME_16;
  case AL_PROFILE_HEVC_MAIN12: return HEVCProfileType::HEVC_PROFILE_MAIN_12;
  case AL_PROFILE_HEVC_MAIN_422: return HEVCProfileType::HEVC_PROFILE_MAIN_422;
  case AL_PROFILE_HEVC_MAIN_422_10: return HEVCProfileType::HEVC_PROFILE_MAIN_422_10;
  case AL_PROFILE_HEVC_MAIN_422_12: return HEVCProfileType::HEVC_PROFILE_MAIN_422_12;
  case AL_PROFILE_HEVC_MAIN_444: return HEVCProfileType::HEVC_PROFILE_MAIN_444;
  case AL_PROFILE_HEVC_MAIN_444_10: return HEVCProfileType::HEVC_PROFILE_MAIN_444_10;
  case AL_PROFILE_HEVC_MAIN_444_12: return HEVCProfileType::HEVC_PROFILE_MAIN_444_12;
  case AL_PROFILE_HEVC_MAIN_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_INTRA;
  case AL_PROFILE_HEVC_MAIN10_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_10_INTRA;
  case AL_PROFILE_HEVC_MAIN12_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_12_INTRA;
  case AL_PROFILE_HEVC_MAIN_422_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_422_INTRA;
  case AL_PROFILE_HEVC_MAIN_422_10_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_422_10_INTRA;
  case AL_PROFILE_HEVC_MAIN_422_12_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_422_12_INTRA;
  case AL_PROFILE_HEVC_MAIN_444_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_444_INTRA;
  case AL_PROFILE_HEVC_MAIN_444_10_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_444_10_INTRA;
  case AL_PROFILE_HEVC_MAIN_444_12_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_444_12_INTRA;
  case AL_PROFILE_HEVC_MAIN_444_16_INTRA: return HEVCProfileType::HEVC_PROFILE_MAIN_444_16_INTRA;
  case AL_PROFILE_HEVC_MAIN_444_STILL: return HEVCProfileType::HEVC_PROFILE_MAIN_444_STILL;
  case AL_PROFILE_HEVC_MAIN_444_16_STILL: return HEVCProfileType::HEVC_PROFILE_MAIN_444_16_STILL;
  default: return HEVCProfileType::HEVC_PROFILE_MAX_ENUM;
  }

  return HEVCProfileType::HEVC_PROFILE_MAX_ENUM;
}

AL_EProfile ConvertModuleToSoftHEVCProfile(HEVCProfileType const& profile)
{
  switch(profile)
  {
  case HEVCProfileType::HEVC_PROFILE_MAIN:
  case HEVCProfileType::HEVC_PROFILE_MAIN_HIGH_TIER: return AL_PROFILE_HEVC_MAIN;
  case HEVCProfileType::HEVC_PROFILE_MAIN_10:
  case HEVCProfileType::HEVC_PROFILE_MAIN_10_HIGH_TIER: return AL_PROFILE_HEVC_MAIN10;
  case HEVCProfileType::HEVC_PROFILE_MAIN_STILL:
  case HEVCProfileType::HEVC_PROFILE_MAIN_STILL_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_STILL;
  case HEVCProfileType::HEVC_PROFILE_MONOCHROME:
  case HEVCProfileType::HEVC_PROFILE_MONOCHROME_HIGH_TIER: return AL_PROFILE_HEVC_MONO;
  case HEVCProfileType::HEVC_PROFILE_MONOCHROME_10:
  case HEVCProfileType::HEVC_PROFILE_MONOCHROME_10_HIGH_TIER: return AL_PROFILE_HEVC_MONO10;
  case HEVCProfileType::HEVC_PROFILE_MONOCHROME_12:
  case HEVCProfileType::HEVC_PROFILE_MONOCHROME_12_HIGH_TIER: return AL_PROFILE_HEVC_MONO12;
  case HEVCProfileType::HEVC_PROFILE_MONOCHROME_16:
  case HEVCProfileType::HEVC_PROFILE_MONOCHROME_16_HIGH_TIER: return AL_PROFILE_HEVC_MONO16;
  case HEVCProfileType::HEVC_PROFILE_MAIN_12:
  case HEVCProfileType::HEVC_PROFILE_MAIN_12_HIGH_TIER: return AL_PROFILE_HEVC_MAIN12;
  case HEVCProfileType::HEVC_PROFILE_MAIN_422:
  case HEVCProfileType::HEVC_PROFILE_MAIN_422_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_422;
  case HEVCProfileType::HEVC_PROFILE_MAIN_422_10:
  case HEVCProfileType::HEVC_PROFILE_MAIN_422_10_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_422_10;
  case HEVCProfileType::HEVC_PROFILE_MAIN_422_12:
  case HEVCProfileType::HEVC_PROFILE_MAIN_422_12_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_422_12;
  case HEVCProfileType::HEVC_PROFILE_MAIN_444:
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_444;
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_10:
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_10_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_444_10;
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_12:
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_12_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_444_12;
  case HEVCProfileType::HEVC_PROFILE_MAIN_INTRA:
  case HEVCProfileType::HEVC_PROFILE_MAIN_INTRA_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_INTRA;
  case HEVCProfileType::HEVC_PROFILE_MAIN_10_INTRA:
  case HEVCProfileType::HEVC_PROFILE_MAIN_10_INTRA_HIGH_TIER: return AL_PROFILE_HEVC_MAIN10_INTRA;
  case HEVCProfileType::HEVC_PROFILE_MAIN_12_INTRA:
  case HEVCProfileType::HEVC_PROFILE_MAIN_12_INTRA_HIGH_TIER: return AL_PROFILE_HEVC_MAIN12_INTRA;
  case HEVCProfileType::HEVC_PROFILE_MAIN_422_INTRA:
  case HEVCProfileType::HEVC_PROFILE_MAIN_422_INTRA_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_422_INTRA;
  case HEVCProfileType::HEVC_PROFILE_MAIN_422_10_INTRA:
  case HEVCProfileType::HEVC_PROFILE_MAIN_422_10_INTRA_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_422_10_INTRA;
  case HEVCProfileType::HEVC_PROFILE_MAIN_422_12_INTRA:
  case HEVCProfileType::HEVC_PROFILE_MAIN_422_12_INTRA_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_422_12_INTRA;
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_INTRA:
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_INTRA_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_444_INTRA;
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_10_INTRA:
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_10_INTRA_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_444_10_INTRA;
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_12_INTRA:
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_12_INTRA_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_444_12_INTRA;
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_16_INTRA:
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_16_INTRA_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_444_16_INTRA;
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_STILL:
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_STILL_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_444_STILL;
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_16_STILL:
  case HEVCProfileType::HEVC_PROFILE_MAIN_444_16_STILL_HIGH_TIER: return AL_PROFILE_HEVC_MAIN_444_16_STILL;
  default: return AL_PROFILE_HEVC;
  }

  return AL_PROFILE_HEVC;
}
