// SPDX-FileCopyrightText: © 2024 Allegro DVT <github-ip@allegrodvt.com>
// SPDX-License-Identifier: MIT

#include "omx_component_interface.h"

OMXComponentInterface::~OMXComponentInterface() = default;
