// SPDX-FileCopyrightText: © 2024 Allegro DVT <github-ip@allegrodvt.com>
// SPDX-License-Identifier: MIT

#pragma once

#include "ROIMngr.h"
#include "module_enums.h"

AL_ERoiQuality ConvertModuleToSoftQualityByPreset(QualityType const& quality);
